<?php

namespace FM\Controller;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use Zeedhi\Framework\DTO\Response;
use Zeedhi\Framework\DataSource\DataSet;
use FM\Service\OrdemServService;

class OrdemServController
{
    protected $OrdemServService;

    public function __construct(OrdemServService $OrdemServService)
    {
        $this->OrdemServService = $OrdemServService;
    }

    public function ordemServ(DTO\Request $request, DTO\Response $response)
    {
        $response->addDataSet(new DataSet('ordens', $this->OrdemServService->getOrdemServ(($request->getParameters())['NR'])));
    }

    public function rotaBase(DTO\Request $request, DTO\Response $response)
    {
        $method = $request->getMethod();
        $parametros = $request->getParameters();
        $ordensServico = '';

        if ($method == "POST") {
            $ordensServico = $this->OrdemServService->insertNovaOrdemServico($parametros);
        } else if ($method == "GET") {
            $ordensServico = $this->OrdemServService->getInformacoesOrdemServico();
        }

        $dtsVal = new DataSet('ordensServico', [$ordensServico]);
        $response->addDataSet($dtsVal);
    }

    public function getOrdemServicoEspecifica(DTO\Request $request, DTO\Response $response)
    {
        $parametros = $request->getParameters();

        $ordem = $this->OrdemServService->getOrdemServicoEspecifica($parametros);
        $dtsVal = new DataSet('ordem', $ordem);
        $response->addDataSet($dtsVal);
    }

    public function comentariosOrdemServico(DTO\Request $request, DTO\Response $response)
    {
        $parametros = $request->getParameters();

        $comentario = $this->OrdemServService->comentariosOrdemServico($parametros);
        $dtsVal = new DataSet('comentarios', [$comentario]);
        $response->addDataSet($dtsVal);
    }

    public function consultarRelacionamentoComSolicitacaoServico(DTO\Request $request, DTO\Response $response)
    {
        $parametros = $request->getParameters();

        $solicitacao = $this->OrdemServService->consultarRelacionamentoComSolicitacaoServico($parametros);
        $dtsVal = new DataSet('solicitacao', [$solicitacao]);
        $response->addDataSet($dtsVal);
    }

    public function inserirNovaOrdemServico(DTO\Request $request, DTO\Response $response)
    {
        $parametros = $request->getParameters();
        // debugApi([$parametros]);
        $variavel = $this->OrdemServService->inserirNovaOrdemServico($parametros);
        $dtsVal = new DataSet('novaOrdem', [$variavel]);
        $response->addDataSet($dtsVal);
    }

    public function insertOrdemServico(Request $request)
    {
        try {
            $parametros = $request->getParameters();
            $this->OrdemServService->insertOrdemServico($parametros);
        } catch (\Exception $ex) {
            $this->sendResponse(['error' => $ex->getMessage(), 'code' => $ex->getCode()], 500);
        }
    }

    public function getSolicitacaoServicoEspecifica(DTO\Request $request, DTO\Response $response)
    {
        $parametros = $request->getParameters();

        $ordem = $this->OrdemServService->getSolicitacaoServicoEspecifica($parametros);
        $dtsVal = new DataSet('solicitacao', $ordem);
        $response->addDataSet($dtsVal);
    }

    public function listOrdemServ(DTO\Request $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        $ordemServs = $this->OrdemServService->listOrdemServ($params);
        $dtsVal = new DataSet('OrdemServ', $ordemServs);
        $response->addDataSet($dtsVal);
    }

    public function listOrdemServJoin(DTO\Request $request, DTO\Response $response)
    {
        $params = $request->getParameters();
        $ordemServs = $this->OrdemServService->listOrdemServJoin($params);
        $dtsVal = new DataSet('OrdemServ', $ordemServs);
        $response->addDataSet($dtsVal);
    }
}
