<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class AtivosInsumosFactory {
    use TQuery;

	protected $connection;
	protected $entityManager;

	public function __construct(EntityManager $entityManager) {
		$this->connection = $entityManager->getConnection();
		$this->entityManager = $entityManager;
	}

    /*
     * TAB ATIVOS
     */

     public function getAtivos() {
		$sql = "SELECT DISTINCT FCM_EQUIPAMENTO.NREQUIPAMENTO, FCM_EQUIPAMENTO.CDEQUIPAMENTO, FCM_EQUIPAMENTO.NMEQUIPAMENTO, FCM_CATEGORIAEQUIP.NRCATEGORIAEQUIP,
					FCM_CATEGORIAEQUIP.NMCATEGORIAEQUIP, NVL((SELECT LISTAGG(FCM_TIPOMANUTENCAO.NMTIPOMANUTENCAO, ', ') WITHIN GROUP (ORDER BY FCM_TIPOMANUTENCAO.NMTIPOMANUTENCAO)
															FROM FCM_TIPOMANUTENCAO
														   	WHERE FCM_TIPOMANUTENCAO.NRTIPOMANUTENCAO IN (SELECT FCM_RELINSUMOSEQUIP.NRRELINSUMOSEQUIP
																										   	FROM FCM_RELINSUMOSEQUIP
																										  	WHERE FCM_RELINSUMOSEQUIP.NREQUIPAMENTO = FCM_EQUIPAMENTO.NREQUIPAMENTO)), ' ') as TIPOSMANUTENCAO,
					FCM_EQUIPAMENTO.IDSITUACAO AS IDSITUACAO,
					CASE FCM_EQUIPAMENTO.IDSITUACAO WHEN '1' THEN 'Em Funcionamento' WHEN '2' THEN 'Em Manutenção' WHEN '3' THEN 'Descontinuado' END as SITUACAO,
					CASE WHEN FCM_EQUIPAMENTO.DTULTATU IS NULL THEN ' ' ELSE TO_CHAR(FCM_EQUIPAMENTO.DTULTATU, 'DD/MM/YYYY') END AS DTULTATU
				FROM FCM_EQUIPAMENTO
				JOIN FCM_CATEGORIAEQUIP
				  ON FCM_EQUIPAMENTO.NRCATEGORIAEQUIP = FCM_CATEGORIAEQUIP.NRCATEGORIAEQUIP";

		return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
	}

	public function insertAtivo($parametros) {
		$sql = "INSERT INTO FCM_EQUIPAMENTO (
					NREQUIPAMENTO,		NMEQUIPAMENTO,	CDEQUIPAMENTO,	NRCATEGORIAEQUIP,
					IDSITUACAO,			NRORG,			IDATIVO,		DTINCLUSAO,
					CDOPERINCLUSAO,		NRORGINCLUSAO	
				)
				VALUES (
					:NREQUIPAMENTO,		:NMEQUIPAMENTO, :CDEQUIPAMENTO, :NRCATEGORIAEQUIP,
					:IDSITUACAO,		:NRORG,			'S',			SYSDATE,
					:CDOPERINCLUSAO,	:NRORGINCLUSAO
				)";
		
		$this->connection->executeStatement($sql, $parametros);
	}

	public function updateAtivo($parametros) {
		$sql = "UPDATE FCM_EQUIPAMENTO
				SET	NMEQUIPAMENTO	 = :NMEQUIPAMENTO,
					CDEQUIPAMENTO	 = :CDEQUIPAMENTO,
					NRCATEGORIAEQUIP = :NRCATEGORIAEQUIP,
					IDSITUACAO		 = :IDSITUACAO,
					DTULTATU		 = SYSDATE,
					CDOPERULTATU	 = :CDOPERULTATU,
					NRORGULTATU		 = :NRORGULTATU
				WHERE NREQUIPAMENTO = :NREQUIPAMENTO";

		$this->connection->executeStatement($sql, $parametros);
	}

	public function getNumRelacEquipEscopo($parametros) {
		$sql = "SELECT COUNT(*)
				FROM FCM_EQUIPAMENTO
				WHERE NREQUIPAMENTO = :NREQUIPAMENTO
				AND	NRESCOPOMANUT = :NRESCOPOMANUT";

		return $this->connection->fetchOne($sql, $parametros);
	}

	public function deletRelacEscopoEquip($parametros){
        $sql = "DELETE FROM FCM_RELACESCOPOSEQUIP
               	WHERE NREQUIPAMENTO = :NREQUIPAMENTO";
        
		$this->connection->executeStatement($sql, $parametros);
    }
    
    public function insertRelacEscopoEquip($parametros){
        $sql = "INSERT INTO FCM_RELACESCOPOSEQUIP (RELACESCOPOSEQUIP, NREQUIPAMENTO, NRESCOPOMANUT, NRORG, IDATIVO, DTINCLUSAO)
              	VALUES (:RELACESCOPOSEQUIP, :NREQUIPAMENTO, :NRESCOPOMANUT, :NRORG, 'S', SYSDATE)";
        
		$this->connection->executeStatement($sql, $parametros);
    }

	public function insertImgAtivo($parametros){
        $sql = "INSERT INTO FCM_IMGEQUIPAMENTO (NRIMGEQUIPAMENTO, NREQUIPAMENTO, DSCAMINHO, NRORG, IDATIVO, DSTYPE, NRSIZE)
              	VALUES (:NRIMGEQUIPAMENTO, :NREQUIPAMENTO, :DSCAMINHO, :NRORG, 'S', :DSTYPE, :NRSIZE)";
        
        $this->connection->executeStatement($sql, $parametros);
    }

	public function getFotosAtivo($parametros) {
		$sql = "SELECT NRIMGEQUIPAMENTO, DSCAMINHO
				FROM FCM_IMGEQUIPAMENTO
				WHERE NREQUIPAMENTO = :NREQUIPAMENTO";

		return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
	}

	public function deleteFotoAtivo($parametros) {
		$sql = "DELETE
				FROM FCM_IMGEQUIPAMENTO
				WHERE NRIMGEQUIPAMENTO = :NRIMGEQUIPAMENTO";
		
		$this->connection->executeStatement($sql, $parametros);
	}

	public function deleteAtivos($parametros) {
		$sql = "DELETE 
				FROM FCM_EQUIPAMENTO
				WHERE NREQUIPAMENTO IN (".$parametros.")";

		$this->connection->executeStatement($sql);
	}

	public function getCategEquip() {
		$sql = "SELECT NRCATEGORIAEQUIP, NMCATEGORIAEQUIP
				FROM FCM_CATEGORIAEQUIP";

		return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
	}

	public function getEscopos() {
		$sql = "SELECT NRESCOPOMANUT, NMESCOPOMANUT
				FROM FCM_ESCOPOMANUT";

		return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
	}

    /*
     * TAB CATEGORIAS
     */

     public function getCategorias() {
        $sql = "SELECT DISTINCT FCM_CATEGORIAEQUIP.NRCATEGORIAEQUIP, FCM_CATEGORIAEQUIP.NMCATEGORIAEQUIP,
                      (SELECT LISTAGG(FCM_CATEGORIAMANUT.NMCATEGORIAMANUT, ', ') WITHIN GROUP (ORDER BY FCM_CATEGORIAMANUT.NMCATEGORIAMANUT)
                                               FROM FCM_CATEGORIAMANUT
                                               JOIN FCM_RELCATEGORIAEQUIPMANUT
                                                ON FCM_CATEGORIAMANUT.NRCATEGORIAMANUT = FCM_RELCATEGORIAEQUIPMANUT.NRCATEGORIAMANUT
                                               WHERE FCM_CATEGORIAEQUIP.NRCATEGORIAEQUIP = FCM_RELCATEGORIAEQUIPMANUT.NRCATEGORIAEQUIP) AS NMTIPOMANUTENCAO,
                      (SELECT LISTAGG(FCM_CATEGORIAMANUT.NRCATEGORIAMANUT, ',') WITHIN GROUP (ORDER BY FCM_CATEGORIAMANUT.NMCATEGORIAMANUT)
                                               FROM FCM_CATEGORIAMANUT
                                               JOIN FCM_RELCATEGORIAEQUIPMANUT
                                                ON FCM_CATEGORIAMANUT.NRCATEGORIAMANUT = FCM_RELCATEGORIAEQUIPMANUT.NRCATEGORIAMANUT
                                               WHERE FCM_CATEGORIAEQUIP.NRCATEGORIAEQUIP = FCM_RELCATEGORIAEQUIPMANUT.NRCATEGORIAEQUIP) AS NRTIPOMANUTENCAO,
                      (SELECT LISTAGG(FCM_EQUIPAMENTO.NMEQUIPAMENTO, ',') WITHIN GROUP (ORDER BY FCM_EQUIPAMENTO.NMEQUIPAMENTO)
                                               FROM FCM_EQUIPAMENTO
                                               WHERE FCM_CATEGORIAEQUIP.NRCATEGORIAEQUIP = FCM_EQUIPAMENTO.NRCATEGORIAEQUIP) AS NMEQUIPAMENTO,
                      (SELECT LISTAGG(FCM_EQUIPAMENTO.NREQUIPAMENTO, ',') WITHIN GROUP (ORDER BY FCM_EQUIPAMENTO.NMEQUIPAMENTO)
                                               FROM FCM_EQUIPAMENTO
                                               WHERE FCM_CATEGORIAEQUIP.NRCATEGORIAEQUIP = FCM_EQUIPAMENTO.NRCATEGORIAEQUIP) AS NREQUIPAMENTO
                FROM FCM_CATEGORIAEQUIP";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, [], [], null, null, null, ['NRTIPOMANUTENCAO']);
    }

    public function insertCategoria($parametros) {
        $sql = "INSERT INTO FCM_CATEGORIAEQUIP (
                    NRCATEGORIAEQUIP,   NMCATEGORIAEQUIP,   NRORG,          IDATIVO,
                    DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO
                )
                VALUES (
                    :NRCATEGORIAEQUIP,  :NMCATEGORIAEQUIP,  :NRORG,         'S',
                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function updateCategoria($parametros) {
        $sql = "UPDATE FCM_CATEGORIAEQUIP
                SET NNMCATEGORIAEQUIP   = :NMCATEGORIAEQUIP,
                    DTULTATU            = SYSDATE
                    CDOPERULTATU        = :CDOPERULTATU,
                    NRORGULTATU         = :NRORGULTATU
                WHERE NRCATEGORIAEQUIP = :NRCATEGORIAEQUIP";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertRelCatManut($parametros) {
        $sql = "INSERT INTO FCM_RELCATEGORIAEQUIPMANUT (NRRELCATEGORIAEQUIPMANUT, NRCATEGORIAEQUIP, NRCATEGORIAMANUT, NRORG, IDATIVO, DTINCLUSAO)
                VALUES (:NRRELCATEGORIAEQUIPMANUT, :NRCATEGORIAEQUIP, :NRCATEGORIAMANUT, :NRORG, 'S', SYSDATE)";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function deleteRelCatEquipManut($parametros) {
        $sql = "DELETE
                FROM FCM_RELCATEGORIAEQUIPMANUT
                WHERE NRCATEGORIAEQUIP IN (".$parametros.")";

        $this->connection->executeStatement($sql);
    }

    public function deleteCategorias($parametros) {
        $sql = "DELETE
                FROM FCM_CATEGORIAEQUIP
                WHERE NRCATEGORIAEQUIP IN (".$parametros.")";

        $this->connection->executeStatement($sql);
    }

    public function getCategManutencao() {
        $sql = "SELECT NRCATEGORIAMANUT, NMCATEGORIAMANUT
                FROM FCM_CATEGORIAMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function getEquip() {
        $sql = "SELECT NREQUIPAMENTO, NMEQUIPAMENTO
                FROM FCM_EQUIPAMENTO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    /*
     * TAB INSUMOS
     */

     public function getInsumos() {
        $sql = "SELECT FCM_INSUMO.NRINSUMO, FCM_INSUMO.NMINSUMO, FCM_INSUMO.CDINSUMO,FCM_UNIDADEMEDIDA.NMUNIDADEMEDIDA, FCM_UNIDADEMEDIDA.NMABREVUNIDADEMEDIDA,
                       FCM_UNIDADEMEDIDA.NRUNIDADEMEDIDA, FCM_INSUMO.CDPRODUTO,
                    CASE WHEN FCM_INSUMO.QTESTOQUE IS NULL THEN 0 ELSE FCM_INSUMO.QTESTOQUE END AS QTESTOQUE,
                    CASE WHEN FCM_INSUMO.QTULTIMARETIRADA IS NULL THEN 0 ELSE FCM_INSUMO.QTULTIMARETIRADA END AS QTULTIMARETIRADA
                FROM FCM_INSUMO
                JOIN FCM_UNIDADEMEDIDA
                  ON FCM_INSUMO.NRUNIDADEMEDIDA = FCM_UNIDADEMEDIDA.NRUNIDADEMEDIDA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function insertInsumo($parametros) {
		$sql = "INSERT INTO FCM_INSUMO (
					NRINSUMO,           NMINSUMO,       CDINSUMO,   CDPRODUTO,
					NRUNIDADEMEDIDA,    NRORG,			IDATIVO,	DTINCLUSAO,
					CDOPERINCLUSAO,		NRORGINCLUSAO	
				)
				VALUES (
					:NRINSUMO,		    :NMINSUMO,      :CDINSUMO,  :CDPRODUTO,
					:NRUNIDADEMEDIDA,	:NRORG,			'S',		SYSDATE,
					:CDOPERINCLUSAO,	:NRORGINCLUSAO
				)";

		$this->connection->executeStatement($sql, $parametros);
	}

    public function updateInsumo($parametros) {
        $sql = "UPDATE FCM_INSUMO
                SET NMINSUMO        = :NMINSUMO,
                    CDINSUMO        = :CDINSUMO,
                    CDPRODUTO       = :CDPRODUTO,
                    NRUNIDADEMEDIDA = :NRUNIDADEMEDIDA,
                    CDOPERULTATU    = :CDOPERULTATU,
                    NRORGULTATU     = :NRORGULTATU
                WHERE NRINSUMO = :NRINSUMO";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function deleteInsumos($parametros) {
        $sql = "DELETE
                FROM FCM_INSUMO
                WHERE NRINSUMO IN (".$parametros.")";
        
        $this->connection->executeStatement($sql);
    }

    public function getUniMedida() {
        $sql = "SELECT NRUNIDADEMEDIDA, NMUNIDADEMEDIDA
                FROM FCM_UNIDADEMEDIDA";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function insertImgInsumo($param){
        $sql = "INSERT INTO FCM_IMGINSUMO (NRIMGINSUMO, NRINSUMO, DSCAMINHO, NRORG, IDATIVO, DSTYPE, NRSIZE)
                VALUES (:NRIMGINSUMO, :NRINSUMO, :DSCAMINHO, :NRORG, 'S', :DSTYPE, :NRSIZE)";
        
        $this->connection->executeStatement($sql, $param);
    }

    public function getFotosInsumo($parametros) {
		$sql = "SELECT NRIMGINSUMO, DSCAMINHO
				FROM FCM_IMGINSUMO
				WHERE NRINSUMO = :NRINSUMO";

		return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql, $parametros);
	}

	public function deleteFotoInsumo($parametros) {
		$sql = "DELETE
				FROM FCM_IMGINSUMO
				WHERE NRIMGINSUMO = :NRIMGINSUMO";
		
		$this->connection->executeStatement($sql, $parametros);
	}
}
