<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class CategoriaManutFactory {
    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function insertCategoriaManutencao($parametros) {
        $sql = "INSERT INTO FCM_CATEGORIAMANUT (
                    NRCATEGORIAMANUT,   NMCATEGORIAMANUT,   NMABREVCATEGMANUT,  NRORG,
                    IDATIVO,            DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,
                    DTULTATU,           CDOPERULTATU,       NRORGULTATU
                )
                VALUES (
                    :NRCATEGORIAMANUT,  :NMCATEGORIAMANUT,  :NMABREVCATEGMANUT, :NRORG,
                    'S',                SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,
                    SYSDATE,            :CDOPERULTATU,      :NRORGULTATU
                )";
        
        $this->connection->executeStatement($sql, $parametros);
    }

    public function editCategoriaManutencao($parametros) {
        $sql = "UPDATE FCM_CATEGORIAMANUT
                SET NMCATEGORIAMANUT    = :NMCATEGORIAMANUT,
                    NMABREVCATEGMANUT   = :NMABREVCATEGMANUT,
                    DTULTATU            = SYSDATE,
                    CDOPERULTATU        = :CDOPERULTATU,
                    NRORGULTATU         = :NRORGULTATU
                WHERE NRCATEGORIAMANUT = :NRCATEGORIAMANUT";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function insertFuncionarioCategoriaManut($parametros) {

        $sql = "INSERT INTO FCM_FUNCCATEGORIAMANUT (NRFUNCCATEGORIAMANUT, NRCATEGORIAMANUT,
                NRFUNCIONARIO, NRORG, DTINCLUSAO)
                VALUES (:NRFUNCCATEGORIAMANUT, :NRCATEGORIAMANUT, :NRFUNCIONARIO, :NRORG, SYSDATE)";

        $std = $this->connection->prepare($sql);
        $std->execute($parametros);
    }

    public function getCategoriasManutencao() {

        $sql = "SELECT FCM_CATEGORIAMANUT.NRCATEGORIAMANUT, FCM_CATEGORIAMANUT.NMABREVCATEGMANUT, FCM_CATEGORIAMANUT.NMCATEGORIAMANUT,
                  (SELECT LISTAGG(FCM_FUNCIONARIO.NMFUNCIONARIO, ',') 
                     WITHIN GROUP (ORDER BY FCM_FUNCCATEGORIAMANUT.NRFUNCIONARIO)
                             FROM FCM_FUNCCATEGORIAMANUT
                             JOIN FCM_FUNCIONARIO
                               ON FCM_FUNCCATEGORIAMANUT.NRFUNCIONARIO = FCM_FUNCIONARIO.NRFUNCIONARIO
                            WHERE FCM_CATEGORIAMANUT.NRCATEGORIAMANUT = FCM_FUNCCATEGORIAMANUT.NRCATEGORIAMANUT) NRFUNCIONARIO
                    FROM FCM_CATEGORIAMANUT
                ORDER BY FCM_CATEGORIAMANUT.NRCATEGORIAMANUT";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);

    }

    public function getTecnicosRegistrados() {

        $sql = "SELECT DISTINCT FUNCAT.NRFUNCIONARIO, FUNC.NMFUNCIONARIO
                    FROM FCM_FUNCCATEGORIAMANUT FUNCAT
                    JOIN FCM_FUNCIONARIO FUNC
                      ON FUNC.NRFUNCIONARIO = FUNCAT.NRFUNCIONARIO";

        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deleteCategoriasManutencao($parametros) {
        $sql = "DELETE
                FROM FCM_CATEGORIAMANUT
                WHERE NRCATEGORIAMANUT IN (".$parametros.")";

        $this->connection->executeStatement($sql);
    }
}

?>