<?php

namespace FM\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Model\Traits\TQuery;

class ImpactoManutFactory {
    use TQuery;
    protected $connection;
    protected $entityManager;

    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    public function insertImpacto($parametros) {
        $sql = "INSERT INTO FCM_IMPACTOMANUT (
                    NRIMPACTOMANUT,     NMIMPACTOMANUT,     NRPRIORIDADEMANUTPADR,  NRORG,      IDATIVO,
                    DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,          DTULTATU,   CDOPERULTATU, 
                    NRORGULTATU
                )
                VALUES (
                    :NRIMPACTOMANUT,    :NMIMPACTOMANUT,    :NRPRIORIDADEMANUTPADR, :NRORG,     'S',
                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,         SYSDATE,    :CDOPERULTATU,
                    :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }

    public function editImpacto($parametros) {
        $sql = "UPDATE FCM_IMPACTOMANUT
                SET NMIMPACTOMANUT          = :NMIMPACTOMANUT,
                    NRPRIORIDADEMANUTPADR   = :NRPRIORIDADEMANUTPADR,
                    DTULTATU                = SYSDATE,
                    CDOPERULTATU            = :CDOPERULTATU,
                    NRORGULTATU             = :NRORGULTATU
                WHERE NRIMPACTOMANUT = :NRIMPACTOMANUT";

        $this->connection->executeStatement($sql, $parametros);
    }
    
    public function getImpactosManutencao() {
        
        $sql = "SELECT DISTINCT I.NRIMPACTOMANUT, I.NMIMPACTOMANUT, I.NRPRIORIDADEMANUTPADR, P.NMPRIORIDADEMANUT
                    FROM FCM_IMPACTOMANUT I
                    LEFT JOIN FCM_PRIORIDADEMANUT P
                    ON P.NRPRIORIDADEMANUT = I.NRPRIORIDADEMANUTPADR
                    LEFT JOIN FCM_RELPRIORIDADEIMPACTO R
                    ON R.NRIMPACTOMANUT = I.NRIMPACTOMANUT";
        
        return $this->createAndExecuteQueryAndBuildResponseData($this->entityManager, $sql);
    }

    public function deleteImpactos($parametros) {
        $sql = "DELETE
                FROM FCM_IMPACTOMANUT
                WHERE NRIMPACTOMANUT IN (".$parametros.")";

        $this->connection->executeStatement($sql);
    }

    public function verificaRelImpactoPrioridade($parametros) {
        $sql = "SELECT COUNT(*)
                FROM FCM_RELPRIORIDADEIMPACTO
                WHERE NRIMPACTOMANUT = :NRIMPACTOMANUT AND NRPRIORIDADEMANUT = :NRPRIORIDADEMANUT";

        return $this->connection->fetchOne($sql, $parametros);
    }

    public function insertImpactoRelacionado($parametros) {
        $sql = "INSERT INTO FCM_RELPRIORIDADEIMPACTO (
                    RELPRIORIDADEIMPACTO,   NRPRIORIDADEMANUT,  NRIMPACTOMANUT,     NRORG,
                    IDATIVO,                DTINCLUSAO,         CDOPERINCLUSAO,     NRORGINCLUSAO,
                    DTULTATU,              CDOPERULTATU,       NRORGULTATU
                )
                VALUES (
                    :RELPRIORIDADEIMPACTO,  :NRPRIORIDADEMANUT, :NRIMPACTOMANUT,    :NRORG,
                    'S',                    SYSDATE,            :CDOPERINCLUSAO,    :NRORGINCLUSAO,
                    SYSDATE,                :CDOPERULTATU,      :NRORGULTATU
                )";

        $this->connection->executeStatement($sql, $parametros);
    }
}

?>