<?php

namespace FM\Service;

use FM\Factory\EquipamentoFactory;
use FM\Factory\ImgEquipamentoFactory;
use FM\Factory\RelacescoposequipFactory;
use FM\Service\FileServerService;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class EquipamentoService {

    protected $EquipamentoFactory;
    protected $ImgEquipamentoFactory;
    protected $RelacescoposequipFactory;
    protected $FileServerService;
    protected $Environment;
    protected $newCode;
    protected $connection;
    protected $fileServerApiKeys;
    protected $fileServerUrl;

    public function __construct (EquipamentoFactory $EquipamentoFactory, FileServerService $FileServerService, EntityManager $entityManager, Environment $Environment, NewCode $newCode, $fileServerApiKeys, $fileServerUrl) {
        $this->EquipamentoFactory = $EquipamentoFactory;
        $this->FileServerService = $FileServerService;
        $this->connection = $entityManager->getConnection();
        $this->Environment = $Environment;
        $this->newCode = $newCode;
        $this->fileServerApiKeys = $fileServerApiKeys;
        $this->fileServerUrl = $fileServerUrl;
    }

    public function getAtivos() {
        return $this->EquipamentoFactory->getAtivos();
    }

    public function salvarAtivo($params) {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
            if ($params["NRROW"] == null) {
                $NREQUIPAMENTO = $this->newCode->getCodeByTableName('FCM_EQUIPAMENTO', 20, 1, 10, true);
    
                $parametroAtivo = array(
                    "NREQUIPAMENTO"     => $NREQUIPAMENTO,
                    "NMEQUIPAMENTO"     => $params["NMEQUIPAMENTO"],
                    "CDEQUIPAMENTO"     => $params["CDEQUIPAMENTO"],
                    "NRCATEGORIAEQUIP"  => $params["NRCATEGORIAEQUIP"],
                    "IDSITUACAO"        => $params["IDSITUACAO"],
                    "NRORG"             => $nrorg,
                    "CDOPERINCLUSAO"    => $operador,
                    "NRORGINCLUSAO"     => $nrorg
                );
    
                $this->EquipamentoFactory->insertAtivo($parametroAtivo);
    
                foreach($params["ESCOPOS"] as $escopo) {
                    $RELACESCOPOSEQUIP = $this->newCode->getCodeByTableName('FCM_RELACESCOPOSEQUIP', 20, 1, 10, true);
    
                    $parametroEscopo = array(
                        "RELACESCOPOSEQUIP" => $RELACESCOPOSEQUIP,
                        "NREQUIPAMENTO"     => $NREQUIPAMENTO,
                        "NRESCOPOMANUT"     => $escopo,
                        "NRORG"             => $nrorg
                    );
    
                    $this->EquipamentoFactory->insertRelacEscopoEquip($parametroEscopo);
                }
            } else {
                $NREQUIPAMENTO = $params["NRROW"];

                $parametroAtivo = array(
                    "NREQUIPAMENTO"     => $NREQUIPAMENTO,
                    "NMEQUIPAMENTO"     => $params["NMEQUIPAMENTO"],
                    "CDEQUIPAMENTO"     => $params["CDEQUIPAMENTO"],
                    "NRCATEGORIAEQUIP"  => $params["NRCATEGORIAEQUIP"],
                    "IDSITUACAO"        => $params["IDSITUACAO"],
                    "CDOPERULTATU"      => $operador,
                    "NRORGULTATU"       => $nrorg
                );

                $this->EquipamentoFactory->updateAtivo($parametroAtivo);

                foreach($params["ESCOPOS"] as $escopo) {
                    $parametroRelacionamento = array(
                        "NREQUIPAMENTO" => $NREQUIPAMENTO,
                        "NRESCOPOMANUT" => $escopo
                    );
                    $numRelacionamentos = $this->EquipamentoFactory->getNumRelacEquipEscopo($parametroRelacionamento);

                    if ($numRelacionamentos == 0) {
                        $RELACESCOPOSEQUIP = $this->newCode->getCodeByTableName('FCM_RELACESCOPOSEQUIP', 20, 1, 10, true);
        
                        $parametroEscopo = array(
                            "RELACESCOPOSEQUIP" => $RELACESCOPOSEQUIP,
                            "NREQUIPAMENTO"     => $NREQUIPAMENTO,
                            "NRESCOPOMANUT"     => $escopo,
                            "NRORG"             => $nrorg
                        );
        
                        $this->EquipamentoFactory->insertRelacEscopoEquip($parametroEscopo);
                    }
                }
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
        }
    }

    public function deleteAtivos($params) {
        $parametroAtivo = implode(',', $params["ROWS"]);
        
        $this->EquipamentoFactory->deleteAtivos($parametroAtivo);
    }

    public function insertFotosAtivo($params) {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();

            foreach($params["FOTOS"] as $foto) {
                $img = $this->FileServerService->uploadFoto([$foto], "foto_manutencao_fmt");
                
                $NRIMGEQUIPAMENTO = $this->newCode->getCodeByTableName('FCM_IMGEQUIPAMENTO', 20, 1, 10, true);
                $parametroImagem = array(
                    "NRIMGEQUIPAMENTO"  => $NRIMGEQUIPAMENTO,
                    "NREQUIPAMENTO"     => $params["NREQUIPAMENTO"],
                    "DSCAMINHO"         => $this->fileServerUrl . "/files/" . $img["path"],
                    "NRORG"             => $nrorg,
                    "DSTYPE"            => $img["type"],
                    "NRSIZE"            => $img["size"]
                );

                $this->EquipamentoFactory->insertImgAtivo($parametroImagem);
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
        }
    }

    public function getFotosAtivo($params) {
        $parametroFotos = array(
            "NREQUIPAMENTO" => $params["NREQUIPAMENTO"]
        );

        return $this->EquipamentoFactory->getFotosAtivo($parametroFotos);
    }

    public function deleteFotoAtivo($params) {
        $parametroFoto = array(
            "NRIMGEQUIPAMENTO" => $params["NRIMGEQUIPAMENTO"]
        );

        return $this->EquipamentoFactory->deleteFotoAtivo($parametroFoto);
    }

    public function getCategEquip() {
        return $this->EquipamentoFactory->getCategEquip();
    }

    public function getEscopos() {
        return $this->EquipamentoFactory->getEscopos();
    }
}

?>