<?php

namespace FM\Service;

use FM\Factory\ImpactoManutFactory;
use Teknisa\Libs\Util\Environment;
use Teknisa\Libs\Util\NewCode;
use Doctrine\ORM\EntityManager;

class ImpactoManutService {

    protected $ImpactoManutFactory;
    protected $Environment;
    protected $newCode;

    public function __construct (EntityManager $entityManager, ImpactoManutFactory $ImpactoManutFactory, Environment $Environment, NewCode $newCode) {
        $this->ImpactoManutFactory = $ImpactoManutFactory;
        $this->connection          = $entityManager->getConnection();
        $this->Environment         = $Environment;
        $this->newCode             = $newCode;
    }

    public function saveImpacto($params) {
        $this->connection->beginTransaction();

        try {
            $nrorg = $this->Environment->getNrorg();
            $operador = $this->Environment->getCdOperador();
            
            foreach ($params["ROWS"] as $row) {
                if ($row["NRIMPACTOMANUT"] == 0) {
                    $NRIMPACTOMANUT = $this->newCode->getCodeByTableName("FCM_IMPACTOMANUT", $nrorg, 1, 10, true);
                    $prioridadePadrao = isset($row['NMPRIORIDADEMANUT']) ? $row['NMPRIORIDADEMANUT'] : "";
    
                    $parametrosImpacto = array(
                        "NRIMPACTOMANUT"        => $NRIMPACTOMANUT,
                        "NMIMPACTOMANUT"        => $row['NMIMPACTOMANUT'],
                        "NRPRIORIDADEMANUTPADR" => $prioridadePadrao,
                        "NRORG"                 => $nrorg,
                        "CDOPERINCLUSAO"        => $operador,
                        "NRORGINCLUSAO"         => $nrorg,
                        "CDOPERULTATU"          => $operador,
                        "NRORGULTATU"           => $nrorg          
                    );

                    $this->ImpactoManutFactory->insertImpacto($parametrosImpacto);
                } else {
                    $parametrosImpacto = array(
                        "NRIMPACTOMANUT"        => $row['NRIMPACTOMANUT'],
                        "NMIMPACTOMANUT"        => $row['NMIMPACTOMANUT'],
                        "NRPRIORIDADEMANUTPADR" => $row['NMPRIORIDADEMANUT'],
                        "CDOPERULTATU"          => $operador,
                        "NRORGULTATU"           => $nrorg          
                    );

                    $this->ImpactoManutFactory->editImpacto($parametrosImpacto);
                }

                if (isset($row['NMPRIORIDADEMANUT'])) {
                    $this->criaRelacinamento($row);
                }
            }

            $this->connection->commit();
        } catch (\Exception $ex) {
            $this->connection->rollback();
            throw $ex;
        }
    }

    public function criaRelacinamento($row) {
        $nrorg = $this->Environment->getNrorg();
        $operador = $this->Environment->getCdOperador();

        $parametrosRel = array(
            "NRIMPACTOMANUT"    => $row['NRIMPACTOMANUT'],
            "NRPRIORIDADEMANUT" => $row['NMPRIORIDADEMANUT']
        );

        $retorno = $this->ImpactoManutFactory->verificaRelImpactoPrioridade($parametrosRel);

        if ($retorno == 0) {
            $RELPRIORIDADEIMPACTO = $this->newCode->getCodeByTableName("FCM_RELPRIORIDADEIMPACTO", $nrorg, 1, 10, true);

            $parametrosImpactoRelacionado = array(
                "RELPRIORIDADEIMPACTO"  => $RELPRIORIDADEIMPACTO,
                "NRPRIORIDADEMANUT"     => $row['NMPRIORIDADEMANUT'],
                "NRIMPACTOMANUT"        => $row['NRIMPACTOMANUT'],
                "NRORG"                 => $nrorg,
                "CDOPERINCLUSAO"        => $operador,
                "NRORGINCLUSAO"         => $nrorg,
                "CDOPERULTATU"          => $operador,
                "NRORGULTATU"           => $nrorg 
            );

            $this->ImpactoManutFactory->insertImpactoRelacionado($parametrosImpactoRelacionado);
        }
    }

    public function getImpactosManutencao() {
        return $this->ImpactoManutFactory->getImpactosManutencao();
    }

    public function deleteImpactos($params) {
        $parametro = implode(',', $params["NRS"]);
        $this->ImpactoManutFactory->deleteImpactos($parametro);
    }
}

?>