/* eslint-disable no-console */
import {
  Http, Loader, Metadata, Router,
} from '@zeedhi/core';
import {
  Button, Form, LoadingService, Modal, ModalService, SelectMultiple, Tag, Text, Textarea,
} from '@zeedhi/common';
import { IIterableCarousel, IterableComponentRender } from '@zeedhi/teknisa-components-common';

export class ControllerSolicitacaoDeServico {
  private modalRecusarSolicitacao!: Modal;

  private ControllerAtivosInsumos = Loader.getInstance('ControllerAtivosInsumos');

  private dadosOriginal: any = {};

  public rotaOS: string = `/converterSolicitacaoEmOrdem?${Number(window.location.search.slice(1))}`;

  public async abrirModalRecusarSolicitacao() {
    this.modalRecusarSolicitacao = ModalService.create({
      name: 'modalRecusarSolicitacao',
      persistent: true,
      grid: {
        cols: 4,
        lg: 4,
        md: 4,
        sm: 5,
      },
      children: [
        {
          name: 'HeadModalRecusarSolicitacao',
          component: 'ZdHeader',
          color: 'transparent',
          padless: true,
          elevation: 0,
          leftSlot: [
            {
              name: 'titleText',
              component: 'ZdText',
              text: 'Recusar solicitação',
              cssClass: 'titulo-card',
            },
          ],
          rightSlot: [
            {
              name: 'closeModalButton_modalRecusarSolicitacao',
              component: 'ZdButton',
              icon: true,
              iconName: 'mdi-close',
              small: true,
              events: {
                click: () => this.fecharModallRecusarSolicitacao(),
              },
            },
          ],
        },
        {
          name: 'formDeRecusarSolicitacao',
          component: 'ZdForm',
          cssClass: 'zd-mt-3',
          children: [
            {
              name: 'MotivoDaRecusa',
              component: 'ZdTextarea',
              label: 'Informe o motivo da recusa',
              clearable: false,
              rows: 3,
              grid: {
                cols: 12,
              },
              validations: {
                required: {},
              },
            },
          ],
        },
        {
          name: 'LinhaComBoaoDeCopiaDosModal',
          component: 'ZdCol',
          align: 'end',
          children: [
            {
              name: 'BotaoCancelaCopiarCard',
              component: 'ZdButton',
              label: 'Cancelar',
              outline: true,
              width: '8vw',
              cssClass: 'zd-mr-2',
              events: {
                click: '{{ControllerSolicitacaoDeServico.fecharModallRecusarSolicitacao}}',
              },
            },
            {
              name: 'BotaoCopiarCard',
              component: 'ZdButton',
              label: 'Recusar',
              width: '8vw',
              events: {
                click: '{{ControllerSolicitacaoDeServico.recusarSolicitacao}}',
              },
            },
          ],
        },
      ],
    });
    this.modalRecusarSolicitacao.show();
  }

  public async fecharModallRecusarSolicitacao() {
    await this.modalRecusarSolicitacao!.hide();
    this.modalRecusarSolicitacao?.destroy();
  }

  public pegarSolicitacoes() {
    Http.get('/SolicitacaoServ/Equipamento/CategoriaManut/Funcionario').then((resposta: any) => {
      const dadosSolicitacaoServ = resposta.data.dataset.SolicitacaoServ;
      dadosSolicitacaoServ.forEach((elemento: any) => {
        if (elemento.NRSOLICITACAOSERV < 10) {
          elemento.NRSOLICITACAOSERV = `00${elemento.NRSOLICITACAOSERV}`;
        } else {
          elemento.NRSOLICITACAOSERV = (elemento.NRSOLICITACAOSERV < 100) ? `0${elemento.NRSOLICITACAOSERV}` : elemento.NRSOLICITACAOSERV;
        }
      });
      const nomeCategoria = dadosSolicitacaoServ.map((dados: any) => dados.NMCATEGORIAMANUT);
      Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_IteracaoDeCardSolicitacaoDeServico_NMCATEGORIAMANUT').updateItems(nomeCategoria.filter((nome: string, pos: number) => nomeCategoria.indexOf(nome) === pos));
      const nomes = dadosSolicitacaoServ.map((dados: any) => dados.NMEQUIPAMENTO);
      Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_IteracaoDeCardSolicitacaoDeServico_NMEQUIPAMENTO').updateItems(nomes.filter((nome: string, pos: number) => nomes.indexOf(nome) === pos));
      const funcionarios = dadosSolicitacaoServ.map((dados: any) => dados.NMFUNCIONARIO);
      Metadata.getInstance<SelectMultiple>('CampoParaFiltrar_IteracaoDeCardSolicitacaoDeServico_NMFUNCIONARIO').updateItems(funcionarios.filter((nome: string, pos: number) => funcionarios.indexOf(nome) === pos));
      Metadata.getInstance<IterableComponentRender>('IteracaoDeCardSolicitacaoDeServico').datasource.updateData(dadosSolicitacaoServ);
      this.dadosOriginal.IteracaoDeCardSolicitacaoDeServico = dadosSolicitacaoServ;
      this.ControllerAtivosInsumos.MaisQueUmaPagina({
        component: Metadata.getInstance<Button>('BotaoProximaPagina_IteracaoDeCardSolicitacaoDeServico'),
      });
    });
  }

  public filtrarSolicitacoes(component: any) {
    const data = this.dadosOriginal;
    this.ControllerAtivosInsumos.filtrar(component, data);
  }

  public async pegarSolicitacao() {
    await Http.get('/escopo').then((resposta: any) => {
      Metadata.getInstance<SelectMultiple>('TagsDaSolicitacaoDeServico').datasource.updateData(resposta.data.dataset.escopo);
    });
    await Http.get(`/SolicitacaoServ/Equipamento/CategoriaManut/Funcionario?NRSOLICITACAOSERV=${window.location.search.slice(1)}`).then((resposta: any) => {
      const dadosSolicitacaoServ = resposta.data.dataset.SolicitacaoServ;
      dadosSolicitacaoServ.forEach((elemento: any) => {
        if (elemento.NRSOLICITACAOSERV < 10) {
          elemento.NRSOLICITACAOSERV = `00${elemento.NRSOLICITACAOSERV}`;
        } else {
          elemento.NRSOLICITACAOSERV = (elemento.NRSOLICITACAOSERV < 100) ? `0${elemento.NRSOLICITACAOSERV}` : elemento.NRSOLICITACAOSERV;
        }
      });
      Metadata.getInstance<Text>('DetalhesDaSocitacaoEquipamento').text = dadosSolicitacaoServ[0].NMEQUIPAMENTO;
      Metadata.getInstance<Text>('DetalhesDaSocitacaoTipoDeServico').text = dadosSolicitacaoServ[0].NMCATEGORIAMANUT;
      Metadata.getInstance<Text>('DetalhesDaSocitacaoImpacto').text = dadosSolicitacaoServ[0].NMIMPACTOMANUT;
      Metadata.getInstance<Text>('DetalhesDaSocitacaoDataDaSolicitacao').text = dadosSolicitacaoServ[0].DTSOLICITACAOSERV;
      Metadata.getInstance<Text>('DetalhesDaSocitacaoSolicitante').text = dadosSolicitacaoServ[0].NMFUNCIONARIO;
      Metadata.getInstance<Text>('DetalhesDaSocitacao').text = dadosSolicitacaoServ[0].DSSOLICITACAOSERV;
      // dadosSolicitacaoServ[0].DSCAMINHO = 'https://ichef.bbci.co.uk/news/800/cpsprodpb/F403/production/_109476426_jheison3.png,https://ichef.bbci.co.uk/news/800/cpsprodpb/F403/production/_109476426_jheison3.png';
      if (dadosSolicitacaoServ[0].DSCAMINHO) {
        Metadata.getInstance<IIterableCarousel>('carousel-com_imagens_solicitacao').iterable!.datasource!.updateData(dadosSolicitacaoServ[0].DSCAMINHO.split(',').map((imagem: any, pos: number) => ({ id: pos, src: imagem })));
      } else {
        Metadata.getInstance<Tag>('tagSpanIterable').isVisible = false;
        Metadata.getInstance<Tag>('tagSpanNoData').isVisible = true;
      }
      if (dadosSolicitacaoServ[0].NRESCOPOMANUT) {
        Metadata.getInstance<SelectMultiple>('TagsDaSolicitacaoDeServico').setValue(dadosSolicitacaoServ[0].NRESCOPOMANUT.split('_').map((nr: any) => Number(nr)));
      }
    });
  }

  public pegarComentarios() {
    Http.get(`/comentarios?NR=${window.location.search.slice(1)}`).then((resposta: any) => {
      Metadata.getInstance<IterableComponentRender>('iterable-cards-novaOrdemServico').datasource.updateData(resposta.data.dataset.comentarios);
      this.ControllerAtivosInsumos.MaisQueUmaPagina({
        component: Metadata.getInstance<Button>('BotaoProximaPagina_iterable-cards-novaOrdemServico'),
      });
    });
  }

  public salvarComentarios() {
    const comentario = Metadata.getInstance<Textarea>('nome-adicionar');

    if (comentario.value.replaceAll(' ', '') !== '') {
      LoadingService.show('Salvando ...');
      Http.post('/comentarios', {
        NRSOLICITACAOSERV: window.location.search.slice(1),
        NRFUNCCOMENTARIO: localStorage.getItem('UserLogado'),
        DTCOMENTARIO: new Date().toLocaleDateString(),
        DSCOMENTARIO: comentario.value,
      }).then((resposta: any) => {
        if (resposta.data.dataset.comentarios[0] === 'OK') {
          this.ControllerAtivosInsumos.mensagemSucesso('Comentário realizado com sucesso!');
          comentario.value = '';
          this.pegarComentarios();
        } else {
          this.ControllerAtivosInsumos.mensagemErro('ao tentar enviar seu comentario');
        }
        LoadingService.hide();
      });
    }
  }

  public recusarSolicitacao() {
    const form = Metadata.getInstance<Form>('formDeRecusarSolicitacao');
    form.value.MotivoDaRecusa = (form.value.MotivoDaRecusa.replaceAll(' ', '') === '') ? '' : form.value.MotivoDaRecusa;
    if (form.validate()) {
      Http.post(`/recusarSolicitacao?NR=${window.location.search.slice(1)}`, {
        NRSOLICITACAOSERV: window.location.search.slice(1),
        NRFUNCCOMENTARIO: localStorage.getItem('UserLogado'),
        DTCOMENTARIO: new Date().toLocaleDateString(),
        DSCOMENTARIO: form.value.MotivoDaRecusa,
      }).then((resposta: any) => {
        if (resposta.data.dataset.SolicitacaoServ[0] === 'OK') {
          this.ControllerAtivosInsumos.mensagemSucesso('Alteração realizado com sucesso!');
          this.fecharModallRecusarSolicitacao();
          Router.replace('/solicitacoes');
        } else {
          this.ControllerAtivosInsumos.mensagemErro('ao tentar enviar sua alteração');
        }
        LoadingService.hide();
      });
    }
  }
}
