import { Utilities } from '@/util/Utilities';
import {
    DialogService,
    Form,
    IGridEvent,
    LoadingService, Modal, ModalService, SelectMultiple, TextInput,
} from '@zeedhi/common';
import { Http, Metadata } from '@zeedhi/core';
import { TekGrid } from '@zeedhi/teknisa-components-common';

export class FmtAtivosInsumosCategorias {
    private modalSalvarCategoria!: Modal;

    private tituloModal: string = '';

    public async abrirModal() {
        this.modalSalvarCategoria = await ModalService.createFromJsonPost('', { jsonPath: '/fmt_ativosInsumos/modal/fmt_ativos_insumos_salvar_categoria' });
        this.modalSalvarCategoria.show();
    }

    public async abrirModalAdicionar() {
        this.tituloModal = 'Adicionar Categoria';
        await this.abrirModal();
    }

    public async abrirModalEdicao({ row }: IGridEvent) {
        this.tituloModal = 'Editar Categoria';
        await this.abrirModal();

        const inputNome = Metadata.getInstance<TextInput>('InputNomeCategoria');
        const selectCategoriasManut = Metadata.getInstance<SelectMultiple>('SelectMultipleCategoriaManutencao');
        const inputNrRow = Metadata.getInstance<TextInput>('NRROW');

        inputNome.value = row.NMCATEGORIAEQUIP;
        selectCategoriasManut.setValue(row.NRTIPOMANUTENCAO.map(Number));
        inputNrRow.value = row.NRCATEGORIAEQUIP;
    }

    public fecharModal() {
        this.modalSalvarCategoria.hide();
        this.modalSalvarCategoria?.destroy();
    }

    public salvarCategoria() {
        const form = Metadata.getInstance<Form>('form-salvar-categoria');
        const grid = Metadata.getInstance<TekGrid>('tek-grid-categorias');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/salvarCategoria', {
                NRROW: form.value.NRROW,
                NMCATEGORIAEQUIP: form.value.InputNomeCategoria,
                CATEGORIAS: form.value.SelectMultipleCategoriaManutencao,
            }).then(() => {
                Utilities.showSuccessMessage();
                grid.datasource.get();
                this.fecharModal();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                LoadingService.hide();
            });
        }
    }

    public messageConfirmDelete({ event }: any) {
        Utilities.showConfirmDeleteMessage(event, 'FmtAtivosInsumosCategorias.deleteCategorias', 'Excluir Categorias', 'Deseja realmente excluir os registros selecionados?');
    }

    public deleteCategorias() {
        const grid = Metadata.getInstance<TekGrid>('tek-grid-categorias');

        LoadingService.show();
        Http.post('deleteCategorias', {
            ROWS: grid.selectedRows.map((el) => el.NRCATEGORIAEQUIP),
        }).then(() => {
            Utilities.showSuccessMessage();
            grid.datasource.get();
        }).catch((error) => {
            Utilities.showErrorMessage(error);
        }).finally(() => {
            grid.selectedRows = [];
            DialogService.hide();
            LoadingService.hide();
        });
    }
}
