import { Utilities } from '@/util/Utilities';
import {
    FileInput,
    Form, LoadingService, Modal, ModalService, Select, TextInput,
} from '@zeedhi/common';
import { Http, IDictionary, Metadata } from '@zeedhi/core';
import { IterableCarousel, IterableComponentRender, TekGrid } from '@zeedhi/teknisa-components-common';
import { ZdText } from '@zeedhi/vuetify';
import { FileUpload } from './FileUpload';

export class FmtSolicitacoesServicos {
    private modalRecusarSolicitacao!: Modal;

    private modalCriarOrdemServico!: Modal;

    private modalCriarSolicitacaoServico!: Modal;

    private visibilidade: boolean = true;

    private currentRow: any;

    private tituloModalRecusar: string = '';

    private rowsRecusar: any;

    private carouselSolicitacaoVisivel: boolean = false;

    private isMountedComentarios: boolean = false;

    get visibilidadeFrame() {
        return !this.visibilidade;
    }

    public mostrarDetalhesSolicitacao({ row }: any) {
        const inpEquipamento = Metadata.getInstance<TextInput>('input-equipamento-tab-detalhes');
        const inpTipoServico = Metadata.getInstance<TextInput>('input-tipo-servico-tab-detalhes');
        const inpImpacto = Metadata.getInstance<TextInput>('input-impacto-tab-detalhes');
        const inpDataSolicitacao = Metadata.getInstance<TextInput>('input-data-solicitacao-tab-detalhes');
        const inpSolicitante = Metadata.getInstance<TextInput>('input-solicitante-tab-detalhes');
        const inpDetalhes = Metadata.getInstance<TextInput>('input-detalhes-tab-detalhes');
        const inpEscopo = Metadata.getInstance<TextInput>('input-escopo-tab-detalhes');
        this.visibilidade = false;
        this.currentRow = row;

        inpEquipamento.value = row.NMEQUIPAMENTO;
        inpTipoServico.value = row.NMCATEGORIAMANUT;
        inpImpacto.value = row.NMIMPACTOMANUT;
        inpDataSolicitacao.value = row.DTSOLICITACAOSERV;
        inpSolicitante.value = row.NMFUNCIONARIO;
        inpDetalhes.value = row.DSSOLICITACAOSERV;
        inpEscopo.value = row.NMESCOPOMANUT;

        setTimeout(() => {
            const carouselFotos = Metadata.getInstance<IterableCarousel>('carousel-fotos-solicitacao');
            this.carregarFotosSolicitacao({ component: carouselFotos });
            if (this.isMountedComentarios) {
                const cardsComentarios = Metadata.getInstance<IterableComponentRender>('iterable-cards-comentarios');
                this.carregarComentariosSolicitacao({ component: cardsComentarios });
            }
        }, 300);
    }

    public montaDadosComentarios() {
        const cardsComentarios = Metadata.getInstance<IterableComponentRender>('iterable-cards-comentarios');
        this.carregarComentariosSolicitacao({ component: cardsComentarios });
        this.isMountedComentarios = true;
    }

    public carregarFotosSolicitacao({ component }: any) {
        this.getFotosSolicitacao(this.currentRow.NRSOLICITACAOSERV).then((response: any) => {
            const txtSemRes = Metadata.getInstance<ZdText>('sem-resultado-imagens-solicitacao');
            if (response.data.data.length > 0) {
                component.iterable.datasource.data = [];
                response.data.data.forEach((dados: IDictionary) => {
                    component.iterable.datasource.data.push({
                        NRIMGSOLICITACAOSERV: dados.NRIMGSOLICITACAOSERV,
                        DSCAMINHO: dados.DSCAMINHO,
                    });
                });
                txtSemRes.isVisible = false;
                component.isVisible = true;
            } else {
                txtSemRes.isVisible = true;
                component.isVisible = false;
            }
        }).catch((error: any) => {
            Utilities.showErrorMessage(error);
        });
    }

    public getFotosSolicitacao(NRROW: any) {
        return Http.post('/getFotosSolicitacao', {
            NRSOLICITACAOSERV: NRROW,
        });
    }

    public carregarComentariosSolicitacao({ component }: any) {
        this.getComentariosSolicitacao(this.currentRow.NRSOLICITACAOSERV).then((response: any) => {
            component.datasource.data = [];
            if (response.data.data.length > 0) {
                response.data.data.forEach((dados: IDictionary) => {
                    component.datasource.data.push({
                        NRCOMENTSOLICITSERV: dados.NRCOMENTSOLICITSERV,
                        NMFUNCIONARIO: dados.NMFUNCIONARIO,
                        DTINCLUSAO: dados.DTINCLUSAO,
                        DSCOMENTARIO: dados.DSCOMENTARIO,
                    });
                });
            }
        });
    }

    public getComentariosSolicitacao(NRROW: any) {
        return Http.post('/getComentariosSolicitacao', {
            NRSOLICITACAOSERV: NRROW,
        });
    }

    public saveComentarioSolicitacao() {
        const form = Metadata.getInstance<Form>('form-comentario-tab');

        if (Utilities.validateForm(form)) {
            Http.post('/insertComentarioSolicitacao', {
                NRSOLICITACAOSERV: this.currentRow.NRSOLICITACAOSERV,
                DSCOMENTARIO: form.value.textareaComentarios,
            }).then(() => {
                Utilities.showSuccessMessage();
                const cardsComentarios = Metadata.getInstance<IterableComponentRender>('iterable-cards-comentarios');
                this.carregarComentariosSolicitacao({ component: cardsComentarios });
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            });
        }
    }

    public voltarTelaSolicitacoes() {
        this.visibilidade = true;
    }

    public async abrirModalRecusarSolicitacao() {
        this.modalRecusarSolicitacao?.destroy();
        this.modalRecusarSolicitacao = await ModalService.createFromJsonPost('', { jsonPath: 'fmt_solicitacoes_servicos/modal/fmt_recusar_solicitacao' });
        this.modalRecusarSolicitacao.show();
    }

    public fecharModalRecusarSolicitacao() {
        this.modalRecusarSolicitacao?.destroy();
        this.modalRecusarSolicitacao.hide();
    }

    public recusarSolicitacao({ row }: any) {
        this.tituloModalRecusar = 'Recusar Solicitação';
        this.rowsRecusar = [row];
        this.abrirModalRecusarSolicitacao();
    }

    public recusarSolicitacoes() {
        const grid = Metadata.getInstance<TekGrid>('grid-solicitacoes-servicos');
        this.tituloModalRecusar = 'Recusar Solicitações';

        if (grid.selectedRows.length > 0) {
            this.rowsRecusar = grid.selectedRows;
            this.abrirModalRecusarSolicitacao();
        } else {
            Utilities.showDialog('Aviso', 'warning', 'Selecione pelo menos um registro!');
        }
    }

    public saveRecusarSolicitacoes() {
        const form = Metadata.getInstance<Form>('form-recusar-solicitacao');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/saveRecusarSolicitacoes', {
                ROWS: this.rowsRecusar,
                DSCOMENTARIO: form.value.textareaMotivoRecusa,
            }).then(() => {
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.fecharModalRecusarSolicitacao();
                LoadingService.hide();
            });
        }
    }

    public async criarOrdemServico({ row }: any) {
        this.modalCriarOrdemServico?.destroy();
        this.modalCriarOrdemServico = await ModalService.createFromJsonPost('', { jsonPath: 'fmt_solicitacoes_servicos/modal/fmt_criar_ordem_servico' });
        this.modalCriarOrdemServico.show();
        this.currentRow = row;
    }

    public fecharOrdemServico() {
        this.modalCriarOrdemServico?.destroy();
        this.modalCriarOrdemServico.hide();
    }

    public expandirDadosOrdemServico({ component }: any) {
        const form = Metadata.getInstance<Form>(`form-${component.name}`);
        form.isVisible = !form.isVisible;
        component.iconName = (component.iconName === 'mdi-chevron-down') ? 'mdi-chevron-right' : 'mdi-chevron-down';
    }

    public preencheDetalhesOrdemServico() {
        const inpNrSolicitacao = Metadata.getInstance<TextInput>('input-nr-solicitacao');
        const inpDataSolicitacao = Metadata.getInstance<TextInput>('input-data-solicitacao');
        const inpSolicitante = Metadata.getInstance<TextInput>('input-solicitante');
        const inpImpacto = Metadata.getInstance<TextInput>('input-impacto');
        const selectEquipamento = Metadata.getInstance<Select>('selectEquipamento');

        inpNrSolicitacao.value = this.currentRow.NRSOLICITACAOSERV;
        inpDataSolicitacao.value = this.currentRow.DTSOLICITACAOSERV;
        inpSolicitante.value = this.currentRow.NMFUNCIONARIO;
        inpImpacto.value = this.currentRow.NMIMPACTOMANUT;
        selectEquipamento.setValue(this.currentRow.NREQUIPAMENTO);
    }

    public visibilidadeTecnicos({ component }: any) {
        const selectTecnicos = Metadata.getInstance<TextInput>('inputTecnicos');
        selectTecnicos.isVisible = component.value === 'N';
    }

    public saveOrdemServico() {
        const form = Metadata.getInstance<Form>('form-informacoes-ordem-servico');
        const grid = Metadata.getInstance<TekGrid>('grid-solicitacoes-servicos');

        if (Utilities.validateForm(form)) {
            LoadingService.show();
            Http.post('/saveOrdemServico', {
                NMORDEMOSERV: form.value.inputNome,
                NREQUIPAMENTO: form.value.selectEquipamento,
                NRCATEGORIAMANUT: form.value.selectCategoriaManutencao,
                NRTIPOMANUTENCAO: form.value.selectTipoManutencao,
                NRPRIORIDADEMANUT: form.value.selectPrioridade,
                IDSITUACAO: form.value.selectStatus,
                DTORDEMSERV: form.value.dateData,
                IDFUNCEXTERNO: form.value.selectProfissionalExterno,
                TECNICOS: form.value.selectTecnicos,
                DESCRICAO: form.value.inputDescricao,
                NRESCOPOS: form.value.selectEscopo,
                SOLICITACAO: true,
            }).then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.fecharOrdemServico();
                LoadingService.hide();
            });
        }
    }

    public async abrirModalCriarSolicitacaoServico() {
        this.modalCriarSolicitacaoServico?.destroy();
        this.modalCriarSolicitacaoServico = await ModalService.createFromJsonPost('', { jsonPath: 'fmt_solicitacoes_servicos/modal/fmt_criar_solicitacao_servico' });
        this.modalCriarSolicitacaoServico.show();
    }

    public fecharModalCriarSolicitacaoServico() {
        this.modalCriarSolicitacaoServico?.destroy();
        this.modalCriarSolicitacaoServico.hide();
    }

    public async saveSolicitacaoServico() {
        const form = Metadata.getInstance<Form>('form-criar-solicitacao-servico');
        const imgs = await this.updloadPictures('file-input-fotos-solicitacao');
        const grid = Metadata.getInstance<TekGrid>('grid-solicitacoes-servicos');

        LoadingService.show();
        if (Utilities.validateForm(form)) {
            Http.post('/saveSolicitacaoServico', {
                NMSOLICITACAOSERV: form.value.inputNomeSolicitacao,
                NRCATEGORIAMANUT: form.value.inputTipoServico,
                NREQUIPAMENTO: form.value.inputEquipamento,
                NRIMPACTOMANUT: form.value.inputImpacto,
                DSSOLICITACAOSERV: form.value.inputDescricao,
                FOTOS: imgs,
            }).then(() => {
                grid.datasource.get();
                Utilities.showSuccessMessage();
            }).catch((error: any) => {
                Utilities.showErrorMessage(error);
            }).finally(() => {
                this.fecharModalCriarSolicitacaoServico();
                LoadingService.hide();
            });
        }
    }

    public async preencheCamposSolicitacao() {
        const inpResponsavel = Metadata.getInstance<Select>('select-responsavel');
        const inpData = Metadata.getInstance<TextInput>('input-data-solicitacao');

        inpResponsavel.setValue(await this.getOperadorLogado());
        inpData.value = (new Date()).toLocaleDateString();
    }

    public mostrarFotoSolicitacao({ component }: any) {
        const carouselFotos = Metadata.getInstance<IterableCarousel>('carouselFotosSolicitacao');
        carouselFotos.iterable.datasource.updateData([]);
        const imagens: any = [];
        component.value.forEach((element: any, pos: any) => {
            imagens.push({
                id: `numero_${pos}`,
                src: window.URL.createObjectURL(element),
            });
        });
        carouselFotos.iterable.datasource.updateData(imagens);
        this.carouselSolicitacaoVisivel = component.value.length > 0;
    }

    public async updloadPictures(name: string) {
        const fileInputObject = Metadata.getInstance<FileInput>(name).value;
        let arquivos: any = [];
        if (fileInputObject && fileInputObject.length > 0) {
            const types: IDictionary = ['image/jpg', 'image/jpeg', 'image/png'];
            const buildFilesPromises = fileInputObject.map((file: any) => {
                if (types.includes(file.type)) {
                    return FileUpload.buildFileSendObject(file);
                }
                return [];
            });
            await Promise.all(buildFilesPromises).then((files: any) => {
                arquivos = files;
            });
          }
        return arquivos;
    }

    public async getOperadorLogado() {
        let cdoper: string = '';
        try {
            const response = await Http.get('/getOperadorLogado');
            cdoper = response.data.CDOPERADOR;
        } catch (error) {
            Utilities.showErrorMessage(error);
        }
        return cdoper;
    }
}
